@@ -25,14 +25,14 @@
 #ifndef NAUTILUS_DIRECTORY_H
 #define NAUTILUS_DIRECTORY_H
 
-#include <gtk/gtkobject.h>
-#include <libgnomevfs/gnome-vfs-types.h>
+#include <gtk/gtk.h>
+#include <gio/gio.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 
 /* NautilusDirectory is a class that manages the model for a directory,
    real or virtual, for Nautilus, mainly the file-manager component. The directory is
    responsible for managing both real data and cached metadata. On top of
-   the file system independence provided by gnome-vfs, the directory
+   the file system independence provided by gio, the directory
    object also provides:
   
        1) A synchronization framework, which notifies via signals as the
@@ -41,16 +41,17 @@
           operations on files.
 */
 
-#define NAUTILUS_TYPE_DIRECTORY \
-	(nautilus_directory_get_type ())
+#define NAUTILUS_TYPE_DIRECTORY nautilus_directory_get_type()
 #define NAUTILUS_DIRECTORY(obj) \
-	(GTK_CHECK_CAST ((obj), NAUTILUS_TYPE_DIRECTORY, NautilusDirectory))
+  (G_TYPE_CHECK_INSTANCE_CAST ((obj), NAUTILUS_TYPE_DIRECTORY, NautilusDirectory))
 #define NAUTILUS_DIRECTORY_CLASS(klass) \
-	(GTK_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_DIRECTORY, NautilusDirectoryClass))
+  (G_TYPE_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_DIRECTORY, NautilusDirectoryClass))
 #define NAUTILUS_IS_DIRECTORY(obj) \
-	(GTK_CHECK_TYPE ((obj), NAUTILUS_TYPE_DIRECTORY))
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NAUTILUS_TYPE_DIRECTORY))
 #define NAUTILUS_IS_DIRECTORY_CLASS(klass) \
-	(GTK_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_DIRECTORY))
+  (G_TYPE_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_DIRECTORY))
+#define NAUTILUS_DIRECTORY_GET_CLASS(obj) \
+  (G_TYPE_INSTANCE_GET_CLASS ((obj), NAUTILUS_TYPE_DIRECTORY, NautilusDirectoryClass))
 
 /* NautilusFile is defined both here and in nautilus-file.h. */
 #ifndef NAUTILUS_FILE_DEFINED
@@ -100,7 +101,8 @@ typedef struct
 	 */
 	void     (* done_loading)        (NautilusDirectory         *directory);
 
-	void     (* load_error)          (NautilusDirectory         *directory);
+	void     (* load_error)          (NautilusDirectory         *directory,
+					  GError                    *error);
 
 	/*** Virtual functions for subclasses to override. ***/
 	gboolean (* contains_file)       (NautilusDirectory         *directory,
@@ -134,6 +136,12 @@ typedef struct
 	 * the list of standard icons (Computer, Home, Trash) on the desktop.
 	 */
 	GList *	 (* get_file_list)	 (NautilusDirectory *directory);
+
+	/* Should return FALSE if the directory is read-only and doesn't
+	 * allow setting of metadata.
+	 * An example of this is the search directory.
+	 */
+	gboolean (* is_editable)         (NautilusDirectory *directory);
 } NautilusDirectoryClass;
 
 /* Basic GObject requirements. */
@@ -144,18 +152,20 @@ GType              nautilus_directory_get_type                 (void);
  * Returns a referenced object, not a floating one. Unref when finished.
  * If two windows are viewing the same uri, the directory object is shared.
  */
-NautilusDirectory *nautilus_directory_get                      (const char                *uri);
+NautilusDirectory *nautilus_directory_get                      (GFile                     *location);
+NautilusDirectory *nautilus_directory_get_by_uri               (const char                *uri);
 NautilusDirectory *nautilus_directory_get_for_file             (NautilusFile              *file);
 
-/* Covers for gtk_object_ref and gtk_object_unref that provide two conveniences:
- * 1) You don't have to cast to GtkObject *, so using these is type safe.
+/* Covers for g_object_ref and g_object_unref that provide two conveniences:
+ * 1) Using these is type safe.
  * 2) You are allowed to call these with NULL,
  */
-void               nautilus_directory_ref                      (NautilusDirectory         *directory);
+NautilusDirectory *nautilus_directory_ref                      (NautilusDirectory         *directory);
 void               nautilus_directory_unref                    (NautilusDirectory         *directory);
 
 /* Access to a URI. */
 char *             nautilus_directory_get_uri                  (NautilusDirectory         *directory);
+GFile *            nautilus_directory_get_location             (NautilusDirectory         *directory);
 
 /* Is this file still alive and in this directory? */
 gboolean           nautilus_directory_contains_file            (NautilusDirectory         *directory,
@@ -208,12 +218,13 @@ gboolean           nautilus_directory_are_all_files_seen       (NautilusDirector
 /* Return true if the directory is local. */
 gboolean           nautilus_directory_is_local                 (NautilusDirectory         *directory);
 
+gboolean           nautilus_directory_is_in_trash              (NautilusDirectory         *directory);
+
 /* Return false if directory contains anything besides a Nautilus metafile.
  * Only valid if directory is monitored. Used by the Trash monitor.
  */
 gboolean           nautilus_directory_is_not_empty             (NautilusDirectory         *directory);
 gboolean           nautilus_directory_file_list_length_reached (NautilusDirectory         *directory);
-char *             nautilus_directory_make_uri_canonical       (const char                *uri);
 
 /* Convenience functions for dealing with a list of NautilusDirectory objects that each have a ref.
  * These are just convenient names for functions that work on lists of GtkObject *.
@@ -227,5 +238,7 @@ GList *            nautilus_directory_list_sort_by_uri         (GList
 /* Fast way to check if a directory is the desktop directory */
 gboolean           nautilus_directory_is_desktop_directory     (NautilusDirectory         *directory);
 
+gboolean           nautilus_directory_is_editable              (NautilusDirectory         *directory);
+
 
 #endif /* NAUTILUS_DIRECTORY_H */
