@@ -29,8 +29,9 @@
 
 #include "nautilus-dnd.h"
 #include "nautilus-directory.h"
+#include "nautilus-file-utilities.h"
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-string.h>
+#include <string.h>
 
 static gboolean
 nautilus_drag_can_accept_files (NautilusFile *drop_target_item)
@@ -60,6 +61,11 @@ nautilus_drag_can_accept_files (NautilusFile *drop_target_item)
 	if (nautilus_file_is_nautilus_link (drop_target_item)) {
 		return TRUE;
 	}
+
+	if (nautilus_is_file_roller_installed () &&
+	    nautilus_file_is_archive (drop_target_item)) {
+		return TRUE;
+	}
 	
 	return FALSE;
 }
@@ -116,6 +122,7 @@ nautilus_drag_can_accept_info (NautilusFile *drop_target_item,
 			return nautilus_drag_can_accept_files (drop_target_item);
 
 		case NAUTILUS_ICON_DND_XDNDDIRECTSAVE:
+		case NAUTILUS_ICON_DND_RAW:
 			return nautilus_drag_can_accept_files (drop_target_item); /* Check if we can accept files at this location */
 
 		case NAUTILUS_ICON_DND_KEYWORD:
@@ -134,11 +141,11 @@ nautilus_drag_can_accept_info (NautilusFile *drop_target_item,
 			g_assert_not_reached ();
 			return FALSE;
 	}
-	
 }
 
 void
-nautilus_drag_file_receive_dropped_keyword (NautilusFile *file, const char *keyword)
+nautilus_drag_file_receive_dropped_keyword (NautilusFile *file,
+					    const char *keyword)
 {
 	GList *keywords, *word;
 
@@ -159,7 +166,7 @@ nautilus_drag_file_receive_dropped_keyword (NautilusFile *file, const char *keyw
 			g_list_free_1 (word);
 		}
 	}
-	
+
 	nautilus_file_set_keywords (file, keywords);
 	eel_g_list_free_deep (keywords);
 }
