@@ -44,6 +44,7 @@
 #include "nautilus-desktop-icon-file.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-debug-log.h"
+#include "nautilus-open-with-dialog.h"
 
 typedef enum {
 	ACTIVATION_ACTION_LAUNCH_DESKTOP_FILE,
@@ -72,6 +73,7 @@ typedef struct {
 	GCancellable *cancellable;
 	GList *locations;
 	GList *mountables;
+	GList *start_mountables;
 	GList *not_mounted;
 	NautilusWindowOpenMode mode;
 	NautilusWindowOpenFlags flags;
@@ -104,6 +106,7 @@ static void cancel_activate_callback                (gpointer            callbac
 static void activate_activation_uris_ready_callback (GList              *files,
 						     gpointer            callback_data);
 static void activation_mount_mountables             (ActivateParameters *parameters);
+static void activation_start_mountables             (ActivateParameters *parameters);
 static void activate_callback                       (GList              *files,
 						     gpointer            callback_data);
 static void activation_mount_not_mounted            (ActivateParameters *parameters);
@@ -287,8 +290,7 @@ NautilusFileAttributes
 nautilus_mime_actions_get_required_file_attributes (void)
 {
 	return NAUTILUS_FILE_ATTRIBUTE_INFO |
-		NAUTILUS_FILE_ATTRIBUTE_LINK_INFO |
-		NAUTILUS_FILE_ATTRIBUTE_METADATA;
+		NAUTILUS_FILE_ATTRIBUTE_LINK_INFO;
 }
 
 static gboolean
@@ -1025,6 +1027,7 @@ activation_parameters_free (ActivateParameters *parameters)
 	g_object_unref (parameters->cancellable);
 	launch_location_list_free (parameters->locations);
 	nautilus_file_list_free (parameters->mountables);
+	nautilus_file_list_free (parameters->start_mountables);
 	nautilus_file_list_free (parameters->not_mounted);
 	g_free (parameters->activation_directory);
 	g_free (parameters->timed_wait_prompt);
@@ -1183,23 +1186,115 @@ get_application_no_mime_type_handler_message (NautilusFile *file, char *uri)
 	return error_message;
 }
 
+static void
+application_selected_cb (NautilusOpenWithDialog *dialog,
+			 GAppInfo *app,
+			 gpointer user_data)
+{
+	GtkWindow *parent_window;
+	NautilusFile *file;
+	GList files;
+
+	parent_window = GTK_WINDOW (user_data);
+	
+	file = g_object_get_data (G_OBJECT (dialog), "mime-action:file");
+
+	files.next = NULL;
+	files.prev = NULL;
+	files.data = file;
+	nautilus_launch_application (app, &files, parent_window);
+}
+
+static void
+choose_program (GtkDialog *message_dialog, int response, gpointer callback_data)
+{
+	GtkWidget *dialog;
+	char *uri;
+	char *mime_type;
+	NautilusFile *file;
+
+	if (response != GTK_RESPONSE_ACCEPT){
+		gtk_widget_destroy (GTK_WIDGET (message_dialog));
+		return;
+	}
+
+	file = g_object_get_data (G_OBJECT (message_dialog), "mime-action:file");
+
+	g_assert (NAUTILUS_IS_FILE (file));
+
+	nautilus_file_ref (file);
+	uri = nautilus_file_get_uri (file);
+	mime_type = nautilus_file_get_mime_type (file);
+
+	dialog = nautilus_open_with_dialog_new (uri, mime_type, NULL);
+	g_object_set_data_full (G_OBJECT (dialog), 
+				"mime-action:file",
+				nautilus_file_ref (file),
+				(GDestroyNotify)nautilus_file_unref);
+	
+	gtk_window_set_screen (GTK_WINDOW (dialog), 
+			       gtk_widget_get_screen (GTK_WIDGET (callback_data)));
+
+	/* Destroy the message dialog after ref:ing the file */
+	gtk_widget_destroy (GTK_WIDGET (message_dialog));
+
+	gtk_widget_show (dialog);
+
+	g_signal_connect_object (dialog, 
+				 "application_selected", 
+				 G_CALLBACK (application_selected_cb),
+				 callback_data,
+				 0);
+			  
+ 	g_free (uri);
+	g_free (mime_type);
+	nautilus_file_unref (file);	
+}
+
 static void
 show_unhandled_type_error (ActivateParametersInstall *parameters)
 {
+	GtkWidget *dialog;
+
 	char *mime_type = nautilus_file_get_mime_type (parameters->file);
 	char *error_message = get_application_no_mime_type_handler_message (parameters->file, parameters->uri);
 	if (g_content_type_is_unknown (mime_type)) {
-		eel_show_error_dialog (error_message,
-				       _("The file is of an unknown type"),
-				       parameters->parent_window);
+		dialog = eel_alert_dialog_new (parameters->parent_window,
+		                      GTK_DIALOG_DESTROY_WITH_PARENT,
+                		      GTK_MESSAGE_ERROR,
+		                      0,
+                		      error_message,
+                		      _("The file is of an unknown type"));
 	} else {
 		char *text;
 		text = g_strdup_printf (_("There is no application installed for %s files"), g_content_type_get_description (mime_type));
-		eel_show_error_dialog (error_message,
-				       text,
-				       parameters->parent_window);
+
+		dialog = eel_alert_dialog_new (parameters->parent_window,
+		                      GTK_DIALOG_DESTROY_WITH_PARENT,
+                		      GTK_MESSAGE_ERROR,
+		                      0,
+                		      error_message,
+                		      text);
+
 		g_free (text);
 	}
+
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Select Application"), GTK_RESPONSE_ACCEPT);
+
+	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+	g_object_set_data_full (G_OBJECT (dialog), 
+				"mime-action:file",
+				nautilus_file_ref (parameters->file),
+				(GDestroyNotify)nautilus_file_unref);
+
+	gtk_widget_show (GTK_WIDGET (dialog));
+	
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (choose_program), parameters->parent_window);
+
 	g_free (error_message);
 	g_free (mime_type);
 }
@@ -1718,8 +1813,7 @@ activate_files (ActivateParameters *parameters)
 
 	flags = parameters->flags;
 	if (count > 1) {
-		if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS) &&
-		    (parameters->flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
+		if ((parameters->flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
 			flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
 		} else {
 			flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
@@ -1966,7 +2060,9 @@ activate_activation_uris_ready_callback (GList *files_ignore,
 		if (nautilus_file_is_broken_symbolic_link (file)) {
 			launch_location_free (location);
 			parameters->locations = g_list_delete_link (parameters->locations, l);
+			pause_activation_timed_cancel (parameters);
 			report_broken_symbolic_link (parameters->parent_window, file);
+			unpause_activation_timed_cancel (parameters);
 			continue;
 		}
 
@@ -2133,7 +2229,85 @@ activation_mount_mountables (ActivateParameters *parameters)
 		return;
 	}
 
-	activation_get_activation_uris (parameters);
+	if (parameters->mountables == NULL && parameters->start_mountables == NULL)
+		activation_get_activation_uris (parameters);
+}
+
+
+static void
+activation_mountable_started (NautilusFile  *file,
+			      GFile         *gfile_of_file,
+			      GError        *error,
+			      gpointer       callback_data)
+{
+	ActivateParameters *parameters = callback_data;
+	LaunchLocation *location;
+
+	/* Remove from list of files that have to be mounted */
+	parameters->start_mountables = g_list_remove (parameters->start_mountables, file);
+	nautilus_file_unref (file);
+
+	if (error == NULL) {
+		/* Remove file */
+		location = find_launch_location_for_file (parameters->locations, file);
+		if (location != NULL) {
+			parameters->locations = g_list_remove (parameters->locations, location);
+			launch_location_free (location);
+		}
+
+	} else {
+		/* Remove failed file */
+		if (error->domain != G_IO_ERROR ||
+		    (error->code != G_IO_ERROR_FAILED_HANDLED)) {
+			location = find_launch_location_for_file (parameters->locations,
+								  file);
+			if (location) {
+				parameters->locations =
+					g_list_remove (parameters->locations,
+						       location);
+				launch_location_free (location);
+			}
+		}
+
+		if (error->domain != G_IO_ERROR ||
+		    (error->code != G_IO_ERROR_CANCELLED &&
+		     error->code != G_IO_ERROR_FAILED_HANDLED)) {
+			eel_show_error_dialog (_("Unable to start location"),
+					       error->message, NULL);
+		}
+
+		if (error->code == G_IO_ERROR_CANCELLED) {
+			activation_parameters_free (parameters);
+			return;
+		}
+	}
+
+	/* Start more mountables */
+	activation_start_mountables (parameters);
+}
+
+static void
+activation_start_mountables (ActivateParameters *parameters)
+{
+	NautilusFile *file;
+	GMountOperation *start_op;
+
+	if (parameters->start_mountables != NULL) {
+		file = parameters->start_mountables->data;
+		start_op = gtk_mount_operation_new (parameters->parent_window);
+		g_signal_connect (start_op, "notify::is-showing",
+				  G_CALLBACK (activate_mount_op_active), parameters);
+		nautilus_file_start (file,
+				     start_op,
+				     parameters->cancellable,
+				     activation_mountable_started,
+				     parameters);
+		g_object_unref (start_op);
+		return;
+	}
+
+	if (parameters->mountables == NULL && parameters->start_mountables == NULL)
+		activation_get_activation_uris (parameters);
 }
 
 /**
@@ -2205,10 +2379,20 @@ nautilus_mime_activate_files (GtkWindow *parent_window,
 			parameters->mountables = g_list_prepend (parameters->mountables,
 								 nautilus_file_ref (file));
 		}
+
+		if (nautilus_file_can_start (file)) {
+			parameters->start_mountables = g_list_prepend (parameters->start_mountables,
+								       nautilus_file_ref (file));
+		}
 	}
 	
 	activation_start_timed_cancel (parameters);
-	activation_mount_mountables (parameters);
+	if (parameters->mountables != NULL)
+		activation_mount_mountables (parameters);
+	if (parameters->start_mountables != NULL)
+		activation_start_mountables (parameters);
+	if (parameters->mountables == NULL && parameters->start_mountables == NULL)
+		activation_get_activation_uris (parameters);
 }
 
 /**
