@@ -163,6 +163,9 @@ action_split_view_same_location_callback (GtkAction *action,
 	window = NAUTILUS_WINDOW (user_data);
 	next_pane = nautilus_window_get_next_pane (window);
 
+	if (!next_pane) {
+		return;
+	}
 	location = nautilus_window_slot_get_location (next_pane->active_slot);
 	if (location) {
 		nautilus_window_slot_go_to (window->details->active_pane->active_slot, location, FALSE);
@@ -261,11 +264,17 @@ action_split_view_callback (GtkAction *action,
 
 	is_active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 	if (is_active != nautilus_navigation_window_split_view_showing (window)) {
+		NautilusWindow *nautilus_window;
+
 		if (is_active) {
 			nautilus_navigation_window_split_view_on (window);
 		} else {
 			nautilus_navigation_window_split_view_off (window);
 		}
+		nautilus_window = NAUTILUS_WINDOW (window);
+		if (nautilus_window->details->active_pane && nautilus_window->details->active_pane->active_slot) {
+			nautilus_view_update_menus (nautilus_window->details->active_pane->active_slot->content_view);
+		}
 	}
 }
 
