@@ -38,6 +38,9 @@
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
 #include <eel/eel-gtk-extensions.h>
+#include <glib/gi18n.h>
+
+#define MENU_ITEM_MAX_WIDTH_CHARS 32
 
 static GtkWindow *bookmarks_window = NULL;
 static NautilusBookmarkList *bookmarks = NULL;
@@ -83,13 +86,13 @@ show_bogus_bookmark_window (NautilusWindow *window,
 			    NautilusBookmark *bookmark)
 {
 	GtkDialog *dialog;
-	char *uri;
+	GFile *location;
 	char *uri_for_display;
 	char *prompt;
 	char *detail;
 
-	uri = nautilus_bookmark_get_uri (bookmark);
-	uri_for_display = eel_format_uri_for_display (uri);
+	location = nautilus_bookmark_get_location (bookmark);
+	uri_for_display = g_file_get_parse_name (location);
 	
 	prompt = _("Do you want to remove any bookmarks with the "
 		   "non-existing location from your list?");
@@ -99,16 +102,17 @@ show_bogus_bookmark_window (NautilusWindow *window,
 					 _("Bookmark for Nonexistent Location"),
 					 GTK_STOCK_CANCEL,
 					 GTK_WINDOW (window));
-	
-	eel_gtk_signal_connect_free_data
-		(GTK_OBJECT (dialog),
-		 "response",
-		 G_CALLBACK (remove_bookmarks_for_uri_if_yes),
-		 g_strdup (uri));
-	
+
+	g_signal_connect_data (dialog,
+			       "response",
+			       G_CALLBACK (remove_bookmarks_for_uri_if_yes),
+			       g_file_get_uri (location),
+			       (GClosureNotify)g_free,
+			       0);
+
 	gtk_dialog_set_default_response (dialog, GTK_RESPONSE_NO);
 
-	g_free (uri);
+	g_object_unref (location);
 	g_free (uri_for_display);
 	g_free (detail);
 }
@@ -136,6 +140,7 @@ nautilus_bookmarks_exiting (void)
 {
 	if (bookmarks_window != NULL) {
 		nautilus_bookmarks_window_save_geometry (bookmarks_window);
+		gtk_widget_destroy (GTK_WIDGET (bookmarks_window));
 	}
 }
 
@@ -149,10 +154,12 @@ void
 nautilus_window_add_bookmark_for_current_location (NautilusWindow *window)
 {
 	NautilusBookmark *bookmark;
+	NautilusWindowSlot *slot;
 
-	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+	g_assert (NAUTILUS_IS_WINDOW (window));
 
-	bookmark = window->current_location_bookmark;
+	slot = window->details->active_pane->active_slot;
+	bookmark = slot->current_location_bookmark;
 
 	if (!nautilus_bookmark_list_contains (nautilus_get_bookmark_list (), bookmark)) {
 		nautilus_bookmark_list_append (nautilus_get_bookmark_list (), bookmark); 
@@ -198,6 +205,24 @@ remove_bookmarks_menu_items (NautilusWindow *window)
 	}
 }
 
+static void
+connect_proxy_cb (GtkActionGroup *action_group,
+                  GtkAction *action,
+                  GtkWidget *proxy,
+                  gpointer dummy)
+{
+	GtkLabel *label;
+
+	if (!GTK_IS_MENU_ITEM (proxy))
+		return;
+
+	label = GTK_LABEL (GTK_BIN (proxy)->child);
+
+	gtk_label_set_use_underline (label, FALSE);
+	gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+	gtk_label_set_max_width_chars (label, MENU_ITEM_MAX_WIDTH_CHARS);
+}
+
 static void
 update_bookmarks (NautilusWindow *window)
 {
@@ -217,6 +242,8 @@ update_bookmarks (NautilusWindow *window)
 	
 	window->details->bookmarks_merge_id = gtk_ui_manager_new_merge_id (ui_manager);
 	window->details->bookmarks_action_group = gtk_action_group_new ("BookmarksGroup");
+	g_signal_connect (window->details->bookmarks_action_group, "connect-proxy",
+			  G_CALLBACK (connect_proxy_cb), NULL);
 
 	gtk_ui_manager_insert_action_group (ui_manager,
 					    window->details->bookmarks_action_group,
@@ -301,9 +328,4 @@ nautilus_window_initialize_bookmarks_menu (NautilusWindow *window)
 	g_signal_connect_object (nautilus_get_bookmark_list (), "contents_changed",
 				 G_CALLBACK (schedule_refresh_bookmarks_menu),
 				 window, G_CONNECT_SWAPPED);
-
-	/* Recreate static & dynamic parts of menu if icon theme changes */
-	g_signal_connect_object (nautilus_icon_factory_get (), "icons_changed",
-				 G_CALLBACK (schedule_refresh_bookmarks_menu),
-				 window, G_CONNECT_SWAPPED);
 }
